// g_client.c

#include "g_local.h"
#include "../common/pmove.h"


static Vec3 player_mins (-16, -16, -24);
static Vec3 player_maxs (16, 16, 32);


void G_PlayerDie (Entity *self, Entity *inflictor, Entity *attacker);

void player_pain (Entity *self, Entity *other);

/*
===========================================================================

USERCMD EXECUTION

===========================================================================
*/

static usercmd_t	cmd;
static Entity *pm_passent;

trace_t G_PMPlayerTrace (const Vec3 &start, const Vec3 &end) {
	return gi.trace (start, player_mins, player_maxs, end, MOVE_NORMAL, pm_passent);
}

bool G_PMTestPlayerPosition (const Vec3 &origin) {
	return gi.TestPlayerPosition (pm_passent, origin);
}

/*
*/
void G_RunCmd (Entity *ent, usercmd_t *ucmd)
{
	Entity		*other;
	int			i;
	int			oldmsec;
	playermove_t	*pmove = (playermove_t *)gi.pmove;

	cmd = *ucmd;

	// chop up very long commands
	if (cmd.msec > 50)
	{
		oldmsec = ucmd->msec;
		cmd.msec = oldmsec/2;
		G_RunCmd (ent, &cmd);
		cmd.msec = oldmsec/2;
		cmd.impulse = 0;
		G_RunCmd (ent, &cmd);
		return;
	}

	if (!ent->fixangle)
		ent->viewangles = ucmd->angles;

	ent->buttons = ucmd->buttons;
	if (ucmd->impulse)
		ent->impulse = ucmd->impulse;

//
// angles
// show 1/3 the pitch angle and all the roll angle	
	if (ent->health > 0)
	{
		if (!ent->fixangle)
		{
			ent->angles[PITCH] = ent->viewangles[PITCH] / 3;
			ent->angles[YAW] = ent->viewangles[YAW];
		}
		ent->angles[ROLL] = 0;
	}

	pmove->origin = ent->origin + (ent->mins - player_mins);
	pmove->velocity = ent->velocity;
	pmove->cmd = *ucmd;
	pmove->s = ent->pmove;

	pm_passent = ent;
	pmove->PointContents = gi.pointcontents;
	pmove->PlayerTrace = G_PMPlayerTrace;
	pmove->TestPlayerPosition = G_PMTestPlayerPosition;

//	movevars.maxspeed = ent->maxspeed;

	gi.PlayerMove ();

	if (pmove->jumped)
		G_Sound (ent, CHAN_BODY, "player/plyrjmp8.wav");

	if (pmove->landspeed < -300)
	{
		if (pmove->landspeed < -650)
		{
			G_Damage (ent, world, world, 5, DT_FALLING); 
			G_Sound (ent, CHAN_VOICE, "player/land2.wav");
		}
		else
			G_Sound (ent, CHAN_VOICE, "player/land.wav");
	}


	ent->origin = pmove->origin - (ent->mins - player_mins);
	ent->velocity = pmove->velocity;
	ent->pmove = pmove->s;
	ent->waterlevel = pmove->waterlevel;
	ent->watertype = pmove->watertype;

	if (pmove->s.pm_flags & PMF_ONGROUND)
		ent->groundentity = (Entity *)pmove->groundent;
	else
		ent->groundentity = NULL;

	if (!ent->spectator)
	{
		G_LinkEntity (ent);
		G_TouchTriggers (ent);

		// touch other objects
		for (i = 0; i < pmove->numtouch; i++) {
			other = (Entity *)pmove->touchents[i];
			if (other->touch)
				other->touch (other, ent);
		}
	}
}

void SpawnClient (Entity *self);

void Respawn (Entity *self)
{
	SpawnClient (self);
}

/*
*/
void G_ClientThink (Entity *self, usercmd_t *cmd)
{
	extern qboolean G_RunThink (Entity *self);
	if (!G_RunThink (self))
		return;

	G_RunCmd (self, cmd);

	if (self->deadflag) {
		//PlayerDeathThink (self);
		if (self->deadflag == DEAD_DEAD) {
			// wait for all buttons released
			if (self->buttons)
				return;
			self->deadflag = DEAD_RESPAWNABLE;
		}

		if (self->deadflag == DEAD_RESPAWNABLE && self->buttons)
			Respawn (self);
		return;
	}

	W_WeaponFrame (self, cmd->buttons, cmd->impulse);
}

/*
** Called when a player begins connecting to the server.
** The game can refuse entrance to a client by returning false.
** If the client is allowed, the connection process will continue
** and eventually get to ClientBegin()
** Changing levels will NOT cause this to be called again, but
** loadgames will.
*/
bool G_ClientConnect (Entity *ent, char *userinfo)
{
	G_ClientUserinfoChanged (ent, userinfo);

	// set spectator
	if (Q_atoi(Info_ValueForKey(userinfo, "spectator")))
		ent->spectator = true;
	else
		ent->spectator = false;

	Com_DPrintf ("Client %s connected\n", ent->netname);

	return true;
}

/*
*/
void G_ClientDisconnect (Entity *ent)
{
	gi.bprintf ("%s dropped\n", ent->netname);
}

/*
** Called whenever the player updates a userinfo variable.
** The game can override any of the settings in place
** (forcing skins or names, etc) before copying it off.
*/
void G_ClientUserinfoChanged (Entity *ent, char *userinfo)
{
	// check for malformed or illegal info strings
//	if (!Info_Validate(userinfo))
//		strcpy (userinfo, "\\name\\badinfo");

	// set name
	strlcpy (ent->netname, Info_ValueForKey(userinfo, "name"), sizeof(ent->netname));
	if (!ent->netname[0])
		strcpy (ent->netname, "unnamed");
}


Entity *SelectSpawnPoint (void)
{
	Entity	*spot;
	int		count = 0;
	int		i;

	// count deathmatch spawn points
	spot = NULL;
	while ((spot = G_Find (spot, FOFS(classname), "info_player_deathmatch")) != NULL)
		count++;

	// select a random point
	if (count) {
		spot = NULL;
		for (i = rand() % count; i; i--)
			spot = G_Find (spot, FOFS(classname), "info_player_deathmatch");
	}

	if (!spot)
		spot = G_Find (spot, FOFS(classname), "info_player_start");

	return spot;
}


void SpawnClient (Entity *self)
{
	self->model = "progs/player.mdl";
	gi.setmodel (self, "progs/player.mdl");
	self->solid = SOLID_BBOX;
	self->movetype = MOVETYPE_WALK;
	self->deadflag = DEAD_NO;
	self->takedamage = DAMAGE_AIM;
	self->mins = player_mins;
	self->maxs = player_maxs;
	self->velocity.Clear();
	self->viewheight = 22;
	self->health = 100;
	self->items = IT_AXE|IT_SHOTGUN;
	self->ammo_shells = 25;
	self->weapon = IT_SHOTGUN;

	if (self->spectator)
		self->pmove.pm_type = PM_SPECTATOR;
	else
		self->pmove.pm_type = PM_NORMAL;

	Entity *spot = SelectSpawnPoint ();
	if (!spot)
		objerror ("No valid spawn points on map\n");

	self->origin = spot->origin;
	self->origin.z += 1;
	self->viewangles = spot->angles;
	self->angles = self->viewangles;	// hmm
	self->fixangle = true;
	self->pmove.steptime = 0;
	self->pmove.landtime = 0;

	W_SetCurrentAmmo (self);

	Vec3 forward;
	void spawn_tfog (const Vec3 &org);
	void spawn_tdeath (const Vec3 &origin, Entity *death_owner);
	AngleVectors (self->angles, forward);
	spawn_tfog (self->origin + forward * 32);
	spawn_tdeath (self->origin, self);

	G_LinkEntity (self);
}

/*
*/
void G_ClientBegin (Entity *ent)
{
	ent->inuse = true;
	ent->classname = "player";
//	ent->origin.Clear();

	ent->die = G_PlayerDie;
	ent->think = NULL;
	ent->pain = player_pain;

	ent->player = !ent->spectator;

	SpawnClient (ent);

	if (!ent->spectator)
		gi.bprintf ("%s entered the game\n", ent->netname);
}

void DeathSound (Entity *self)
{
	// water death sounds
	if (self->waterlevel == 3) {
//		DeathBubbles(self, 5);
		G_Sound (self, CHAN_VOICE, "player/h2odeath.wav");
		return;
	}
	
	static char *deathsounds[5] = {
		"player/death1.wav",
		"player/death2.wav",
		"player/death3.wav",
		"player/death4.wav",
		"player/death5.wav"};

	G_Sound (self, CHAN_VOICE, deathsounds[rand() % 5]);
}


extern Entity *damage_inflictor;

Vec3 VelocityForDamage (Entity *self, float dm)
{
	Vec3 v;

	if (!damage_inflictor->velocity.IsNull())	{
		v = 0.5 * damage_inflictor->velocity;
		Vec3 dir = self->origin - damage_inflictor->origin;
		Normalize (&dir);
		v = v + (25 * dir);
		v.z = 100 + 240 * random();
		v.x = v.x + (200 * crandom());
		v.y = v.y + (200 * crandom());
		//dprint ("Velocity gib\n");                
	}
	else
	{
		v.x = 100 * crandom();
		v.y = 100 * crandom();
		v.z = 200 + 100 * random();
	}

	if (dm > -50) {
		v = v * 0.7;
	}
	else if (dm > -200)	{
		v = v * 2;
	}
	else
		v = v * 10;

	return v;
}

void ThrowGib (Entity *self, char *gibname, float dm)
{
	Entity *gib = G_Spawn();
	gi.setmodel (gib, gibname);
	gib->origin = self->origin;
	gib->velocity = VelocityForDamage (gib, dm);
	gib->movetype = MOVETYPE_BOUNCE;
	gib->solid = SOLID_NOT;
	gib->avelocity[0] = random()*600;
	gib->avelocity[1] = random()*600;
	gib->avelocity[2] = random()*600;
	gib->think = G_FreeEntity;
	gib->ltime = g_time;	// What's this for?
	gib->nextthink = g_time + 10 + random()*10;
	gib->frame = 0;
	gib->flags = 0;
	G_LinkEntity (self);
}

void ThrowHead (Entity *self, char *gibname, float dm)
{
	gi.setmodel (self, gibname);
	self->frame = 0;
	self->nextthink = -1;
	self->movetype = MOVETYPE_BOUNCE;
	self->takedamage = DAMAGE_NO;
	self->solid = SOLID_NOT;
	self->viewheight = 8;
	setsize (self, -16, -16, 0, 16, 16, 56);
	self->velocity = VelocityForDamage (self, dm);
	self->origin.z -= 24;
//	self->flags &= ~FL_ONGROUND;
	self->pmove.pm_flags &= ~PMF_ONGROUND;			// need this?
	self->groundentity = NULL;
	self->avelocity = crandom() * Angles(600, 0, 0);
	G_LinkEntity (self);
}


void GibPlayer (Entity *self)
{
	ThrowHead (self, "progs/h_player.mdl", self->health);
	ThrowGib (self, "progs/gib1.mdl", self->health);
	ThrowGib (self, "progs/gib2.mdl", self->health);
	ThrowGib (self, "progs/gib3.mdl", self->health);

	self->deadflag = DEAD_DEAD;

/*	if (damage_attacker->classname == "teledeath")
	{
		sound (self, CHAN_VOICE, "player/teledth1.wav", 1, ATTN_NONE);
		return;
	}

	if (damage_attacker.classname == "teledeath2")
	{
		sound (self, CHAN_VOICE, "player/teledth1.wav", 1, ATTN_NONE);
		return;
	}
*/
	
	if (rand()&1)
		G_Sound (self, CHAN_VOICE, "player/gib.wav");
	else
		G_Sound (self, CHAN_VOICE, "player/udeath.wav");
}


void player_diea1 (Entity *self);
void player_die_ax1 (Entity *self);

/*
*/
void G_PlayerDie (Entity *self, Entity *inflictor, Entity *attacker)
{
/*	self->invisible_finished = 0;    // don't die as eyes
	self->invincible_finished = 0;
	self->super_damage_finished = 0;
	self->radsuit_finished = 0;
	self->modelindex = modelindex_player;    // don't use eyes
*/

//	DropBackpack();

	self->weaponmodel = "";
	self->viewheight = -8;
	self->deadflag = DEAD_DYING;
	self->solid = SOLID_NOT;
//	self->flags &= ~FL_ONGROUND;
	self->movetype = MOVETYPE_TOSS;
	self->pmove.pm_type = PM_DEAD;
	if (self->velocity[2] < 10)
		self->velocity[2] += random()*300;

	if (self->health < -40)
	{
		GibPlayer (self);
		return;
	}

	DeathSound (self);
	
	self->angles[PITCH] = 0;
	self->angles[ROLL] = 0;
	
	if (self->weapon == IT_AXE)
	{
		player_die_ax1 (self);
		return;
	}

	// start death frames...
	player_diea1 (self);
}

/*
=============================================================================

CLIENT COMMANDS

=============================================================================
*/

void Cmd_Say_f (Entity *self) {
	int argc = gi.argc();
	if (argc < 2)
		return;

	char text[1024];
	strlcpy (text, self->netname, sizeof(text));
	strlcat (text, ": ", sizeof(text));

	for (int i = 1; i < argc; i++)
		strlcat (text, gi.argv(i), sizeof(text));

	// ouch! ouch!
	gi.bprintf ("%s", text);
}

void Cmd_Kill_f (Entity *self) {
	if (self->health <= 0) 	{
		gi.cprintf (self, "Can't suicide -- already dead!\n");
		return;
	}

	// TODO...
}


void Cmd_Give_f (Entity *self) {
	if (!g_cheats->value) {
		gi.cprintf (self, "Cheats are not allowed on this server\n");
		return;
	}
	
	if (gi.argc() < 2)
		return;
	char *t = gi.argv(1);

	if (strequal(t, "all")) {
		self->items |= IT_AXE|IT_SHOTGUN|IT_SUPER_SHOTGUN|IT_NAILGUN|
			IT_SUPER_NAILGUN|IT_ROCKET_LAUNCHER|IT_GRENADE_LAUNCHER|IT_LIGHTNING;
		self->ammo_shells = 100;
		self->ammo_nails = 200;
		self->ammo_rockets = 100;
		self->ammo_cells = 100;
		// give red armor
		self->armortype = 0.8;
		self->armorvalue = 200;
		self->items |= IT_ARMOR3;

		if (self->health < 100)
			self->health = 100;
	}

	int v = atoi(gi.argv(2));
	
	switch (t[0])
	{
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
		self->items |= self->items | IT_SHOTGUN<< (t[0] - '2');
		break;
	
	case 's':
		self->ammo_shells = v;
		break;		
	case 'n':
		self->ammo_nails = v;
		break;		
	case 'r':
		self->ammo_rockets = v;
		break;		
	case 'h':
		self->health = v;
		break;		
	case 'c':
		self->ammo_cells = v;
		break;		
	}
}

void Cmd_God_f (Entity *self) {
	if (!g_cheats->value) {
		gi.cprintf (self, "Cheats are not allowed on this server\n");
		return;
	}

	self->flags ^= FL_GODMODE;
	if (self->flags & FL_GODMODE)
		gi.cprintf (self, "godmode ON\n");
	else
		gi.cprintf (self, "godmode OFF\n");
}


void Cmd_Noclip_f (Entity *self) {
	if (!g_cheats->value) {
		gi.cprintf (self, "Cheats are not allowed on this server\n");
		return;
	}
	if (self->health <= 0) {
		gi.cprintf (self, "You must be alive to use this command.\n");
		return;
	}

	if (self->movetype != MOVETYPE_NOCLIP)
	{
		self->movetype = MOVETYPE_NOCLIP;
		self->pmove.pm_type = PM_SPECTATOR;
		gi.cprintf (self, "noclip ON\n");
	}
	else
	{
		self->movetype = MOVETYPE_WALK;
		self->pmove.pm_type = PM_NORMAL;		// FIXME
		gi.cprintf (self, "noclip OFF\n");
	}
}


void Cmd_Fly_f (Entity *self) {
	if (!g_cheats->value) {
		gi.cprintf (self, "Cheats are not allowed on this server\n");
		return;
	}
	if (self->health <= 0) {
		gi.cprintf (self, "You must be alive to use this command.\n");
		return;
	}

	if (self->movetype != MOVETYPE_FLY)
	{
		self->movetype = MOVETYPE_FLY;
		self->pmove.pm_type = PM_FLY;
		gi.cprintf (self, "fly mode ON\n");
	}
	else
	{
		self->movetype = MOVETYPE_WALK;
		self->pmove.pm_type = PM_NORMAL;		// FIXME
		gi.cprintf (self, "fly mode OFF\n");
	}
}


/*
** Commands issued by the client via 'cmd <command>'
*/
bool G_ClientCommand (Entity *self)
{
	char *cmd = gi.argv (0);

	if (striequal(cmd, "kill"))
		Cmd_Kill_f (self);
	else if (striequal(cmd, "give"))
		Cmd_Give_f (self);
	else if (striequal(cmd, "god"))
		Cmd_God_f (self);
	else if (striequal(cmd, "noclip"))
		Cmd_Noclip_f (self);
	else if (striequal(cmd, "fly"))
		Cmd_Fly_f (self);
	else if (striequal(cmd, "say") || striequal(cmd, "say_team"))		// ouch!
		Cmd_Say_f (self);
	else
		return false;

	return true;
}


/*QUAKED info_player_start (1 0 0) (-16 -16 -24) (16 16 24)
The normal starting point for a level.
*/
void SP_info_player_start (Entity *self)
{
}

/*QUAKED info_player_start2 (1 0 0) (-16 -16 -24) (16 16 24)
Only used on start map for the return point from an episode.
*/
void SP_info_player_start2 (Entity *self)
{
}

/*QUAKED info_player_deathmatch (1 0 1) (-16 -16 -24) (16 16 24)
potential spawning position for deathmatch games
*/
void SP_info_player_deathmatch (Entity *self)
{
}

/*QUAKED info_player_coop (1 0 1) (-16 -16 -24) (16 16 24)
potential spawning position for coop games
*/
void SP_info_player_coop (Entity *self)
{
}

/*QUAKED info_intermission (1 0.5 0.5) (-16 -16 -16) (16 16 16)
This is the camera point for the intermission.
Use mangle instead of angle, so you can set pitch or roll as well as yaw.  'pitch roll yaw'
*/
void SP_info_intermission (Entity *self)
{
	self->angles = self->mangle;
}

/*QUAKED trigger_changelevel (0.5 0.5 0.5) ? NO_INTERMISSION
When the player touches this, he gets sent to the map listed in the "map" variable.  Unless the NO_INTERMISSION flag is set, the view will go to the info_intermission spot and display stats.
*/
void SP_trigger_changelevel (Entity *self)
{
//	if (!self->map)
//		objerror ("changelevel trigger doesn't have map");
	
//	InitTrigger (self);
//	self->touch = changelevel_touch;
}
