// g_items.c

#include "g_local.h"


/*
================
droptofloor
================
*/
void droptofloor (Entity *ent)
{
	Vec3 dest = ent->origin;
	dest.z -= 256;

	trace_t trace = gi.trace (ent->origin, ent->mins, ent->maxs,
								dest, MOVE_NORMAL, ent);

	if (trace.fraction == 1)
	{	// not landed or anything
		gi.dprintf ("droptofloor \"%s\": item fell out of level at '%f %f %f'\n",
			ent->classname, ent->origin[0], ent->origin[1], ent->origin[2]);
		remove (ent);
		return;
	}

	if (trace.startsolid)
	{
		gi.dprintf ("droptofloor \"%s\": item spawned in solid at '%f %f %f'\n",
			ent->classname, ent->origin[0], ent->origin[1], ent->origin[2]);
		remove (ent);
		return;
	}

	ent->origin = trace.endpos;
	ent->groundentity = (Entity *)trace.ent;
	G_LinkEntity (ent);
}

/*
============
PlaceItem

plants the object on the floor
============
*/
void PlaceItem (Entity *ent)
{
	ent->flags = FL_ITEM;				// make extra wide
	ent->solid = SOLID_TRIGGER;
	ent->movetype = MOVETYPE_TOSS;  
	ent->velocity.Clear();
	ent->origin[2] += 6;
	droptofloor (ent);
}

/*
============
StartItem

Sets item flags and plants the object on the floor

Items can't be immediately dropped to floor, because they might
be on an entity that hasn't spawned yet.
============
*/
void StartItem (Entity *ent)
{
	ent->nextthink = g_time + 0.2;		// items start after other solids
	ent->think = PlaceItem;
}

/*
============
RespawnItem
============
*/
void RespawnItem (Entity *self)
{
	gi.setmodel (self, self->model);		// restore original model
	self->solid = SOLID_TRIGGER;		// allow it to be touched again
	G_LinkEntity (self);
	G_Sound (self, CHAN_VOICE, "items/itembk2.wav");    // play respawn sound
}


/*
=========================================================================

HEALTH BOX

=========================================================================
*/

#define H_ROTTEN	1
#define H_MEGA		2

void Megahealth_Think (Entity *self)
{
	Entity *other = self->owner;

	if (other->health > 100)
	{
		other->health = other->health - 1;
		self->nextthink = g_time + 1;
		return;
	}

	other->items &= ~IT_SUPERHEALTH;
	
	if (g_deathmatch->value != 2)    // deathmatch 2 is silly old rules
	{
		self->nextthink = g_time + 20;
		self->think = RespawnItem;
	}
};


void Touch_Health (Entity *self, Entity *other)
{
	if (!other->player || other->health <= 0)
		return;
	
//	if (g_deathmatch->value == 4)
//		if (other.invincible_time > 0)
//			return;

	if (self->healtype == 2)
	{	// megahealth
		if (other->health >= 250)
			return;
		other->health += self->healamount;
		if (other->health > 250)
			other->health = 250;
	} else {
		if (other->health >= 100)
			return;
		other->health += self->healamount;
		if (other->health > 100)
			other->health = 100;
	}
	
/*	sprint(other, PRINT_LOW, "You receive ");
	s = ftos(self.healamount);
	sprint(other, PRINT_LOW, s);
	sprint(other, PRINT_LOW, " health\n"); */
	
// health touch sound
	G_Sound (other, CHAN_ITEM, self->noise);

//	stuffcmd (other, "bf\n");
	
	self->modelindex = 0;		// make it invisible
	self->solid = SOLID_NOT;

	// Megahealth = rot down the player's super health
	if (self->healtype == 2)
	{
		other->items |= IT_SUPERHEALTH;
		if (g_deathmatch->value != 4)
		{
			self->nextthink = g_time + 5;
			self->think = Megahealth_Think;
		}
		self->owner = other;
	}
	else
	{
		if (g_deathmatch->value != 2)            // deathmatch 2 is the silly old rules
		{
			self->nextthink = g_time + 20;
			self->think = RespawnItem;
		}
	}
	
	G_UseTargets (self, other);
}

void SP_item_health (Entity *ent)
{
	if (ent->spawnflags & H_ROTTEN)
	{
		G_PrecacheModel ("maps/b_bh10.bsp");
		gi.setmodel (ent, "maps/b_bh10.bsp");
		G_PrecacheSound ("items/r_item1.wav");
		ent->noise = "items/r_item1.wav";
		ent->healamount = 15;
		ent->healtype = 0;
	}
	else if (ent->spawnflags & H_MEGA)
	{
		G_PrecacheModel ("maps/b_bh100.bsp");
		gi.setmodel (ent, "maps/b_bh100.bsp");
		G_PrecacheSound ("items/r_item2.wav");
		ent->noise = "items/r_item2.wav";
		ent->healamount = 100;
		ent->healtype = 2;
	}
	else
	{
		G_PrecacheModel ("maps/b_bh25.bsp");
		gi.setmodel (ent, "maps/b_bh25.bsp");
		G_PrecacheSound ("items/health1.wav");
		ent->noise = "items/health1.wav";
		ent->healamount = 25;
		ent->healtype = 1;
	}

	setsize (ent, 0, 0, 0, 32, 32, 56);
	ent->touch = Touch_Health;
	StartItem (ent);
}



/*
===============================================================================

ARMOR

===============================================================================
*/

void Touch_Armor (Entity *self, Entity *other)
{
	int		value, bit;
	float	type;
	
	if (!other->player || other->health <= 0)
		return;

/*	if (g_deathmatch->value == 4)
		if (other.invincible_time > 0)
			return;
*/

	if (strequal(self->classname, "item_armor1"))
	{
		type = 0.3;
		value = 100;
		bit = IT_ARMOR1;
	}
	else if (strequal(self->classname, "item_armor2"))
	{
		type = 0.6;
		value = 150;
		bit = IT_ARMOR2;
	}
	else // if (strequal(self->classname, "item_armorInv"))
	{
		type = 0.8;
		value = 200;
		bit = IT_ARMOR3;
	}
	if (other->armortype * other->armorvalue >= type*value)
		return;
		
	other->armortype = type;
	other->armorvalue = value;
	other->items &= ~(IT_ARMOR1 | IT_ARMOR2 | IT_ARMOR3);
	other->items |= bit;

	self->solid = SOLID_NOT;
	self->modelindex = 0;		// make it invisible

	if (g_deathmatch->value != 2)
		self->nextthink = g_time + 20;
	self->think = RespawnItem;

	//sprint(other, PRINT_LOW, "You got armor\n");
// armor touch sound
	G_Sound (other, CHAN_ITEM, "items/armor1.wav");
	//stuffcmd (other, "bf\n");
	
	G_UseTargets (self, other);		// fire all targets / killtargets
}


/*QUAKED item_armor1 (0 .5 .8) (-16 -16 0) (16 16 32)
*/
void SP_item_armor1 (Entity *ent)
{
	G_PrecacheSound ("items/armor1.wav");
	G_PrecacheModel ("progs/armor.mdl");
	gi.setmodel (ent, "progs/armor.mdl");
	ent->skin = 0;
	ent->effects = EF_ROTATE;
	setsize (ent, -16, -16, 0, 16, 16, 56);
	ent->touch = Touch_Armor;
	StartItem (ent);
}

/*QUAKED item_armor2 (0 .5 .8) (-16 -16 0) (16 16 32)
*/
void SP_item_armor2 (Entity *ent)
{
	G_PrecacheSound ("items/armor1.wav");
	G_PrecacheModel ("progs/armor.mdl");
	gi.setmodel (ent, "progs/armor.mdl");
	ent->skin = 1;
	ent->effects = EF_ROTATE;
	setsize (ent, -16, -16, 0, 16, 16, 56);
	ent->touch = Touch_Armor;
	StartItem (ent);
}

/*QUAKED item_armorInv (0 .5 .8) (-16 -16 0) (16 16 32)
*/
void SP_item_armorInv (Entity *ent)
{
	G_PrecacheSound ("items/armor1.wav");
	G_PrecacheModel ("progs/armor.mdl");
	gi.setmodel (ent, "progs/armor.mdl");
	ent->skin = 2;
	ent->effects = EF_ROTATE;
	setsize (ent, -16, -16, 0, 16, 16, 56);
	ent->touch = Touch_Armor;
	StartItem (ent);
}


/*
===============================================================================

WEAPONS

===============================================================================
*/

void bound_other_ammo (Entity *other)
{
	if (other->ammo_shells > 100)
		other->ammo_shells = 100;
	if (other->ammo_nails > 200)
		other->ammo_nails = 200;
	if (other->ammo_rockets > 100)
		other->ammo_rockets = 100;               
	if (other->ammo_cells > 100)
		other->ammo_cells = 100;         
}


int RankForWeapon (int w)
{
	if (w == IT_LIGHTNING)
		return 1;
	if (w == IT_ROCKET_LAUNCHER)
		return 2;
	if (w == IT_SUPER_NAILGUN)
		return 3;
	if (w == IT_GRENADE_LAUNCHER)
		return 4;
	if (w == IT_SUPER_SHOTGUN)
		return 5;
	if (w == IT_NAILGUN)
		return 6;
	return 7;
}

int WeaponCode (int w)
{
	if (w == IT_SUPER_SHOTGUN)
		return 3;
	if (w == IT_NAILGUN)
		return 4;
	if (w == IT_SUPER_NAILGUN)
		return 5;
	if (w == IT_GRENADE_LAUNCHER)
		return 6;
	if (w == IT_ROCKET_LAUNCHER)
		return 7;
	if (w == IT_LIGHTNING)
		return 8;
	return 1;
}

/*
=============
Deathmatch_Weapon

Deathmatch weapon change rules for picking up a weapon

.float          ammo_shells, ammo_nails, ammo_rockets, ammo_cells;
=============
*/
void Deathmatch_Weapon (Entity *self, int oldw, int neww)
{
	float or, nr;

// change self->weapon if desired
	or = RankForWeapon (self->weapon);
	nr = RankForWeapon (neww);
	if ( nr < or )
		self->weapon = neww;
}

void Touch_Weapon (Entity *self, Entity *other)
{
	  int		 hadammo, best, neww, old;
	  qboolean   leave;

	// For client weapon_switch
	  int	w_switch;

	if (!other->player)
		return;

/*	if ((stof(infokey(other,"w_switch"))) == 0)
		w_switch = 8;
	else
		w_switch = stof(infokey(other,"w_switch"));*/
	w_switch = 1;	// @@@
	
// if the player was using his best weapon, change up to the new one if better			
	best = W_BestWeapon(other);

	if (g_deathmatch->value == 2 || g_deathmatch->value == 3 || g_deathmatch->value == 5)
		leave = 1;
	else
		leave = 0;
	
	if (strequal(self->classname, "weapon_nailgun"))
	{
		if (leave && (other->items & IT_NAILGUN) )
			return;
		hadammo = other->ammo_nails;					 
		neww = IT_NAILGUN;
		other->ammo_nails = other->ammo_nails + 30;
	}
	else if (strequal(self->classname, "weapon_supernailgun"))
	{
		if (leave && (other->items & IT_SUPER_NAILGUN) )
			return;
		hadammo = other->ammo_rockets;					 
		neww = IT_SUPER_NAILGUN;
		other->ammo_nails = other->ammo_nails + 30;
	}
	else if (strequal(self->classname, "weapon_supershotgun"))
	{
		if (leave && (other->items & IT_SUPER_SHOTGUN) )
			return;
		hadammo = other->ammo_rockets;					 
		neww = IT_SUPER_SHOTGUN;
		other->ammo_shells = other->ammo_shells + 5;
	}
	else if (strequal(self->classname, "weapon_rocketlauncher"))
	{
		if (leave && (other->items & IT_ROCKET_LAUNCHER) )
			return;
		hadammo = other->ammo_rockets;					 
		neww = IT_ROCKET_LAUNCHER;
		other->ammo_rockets = other->ammo_rockets + 5;
	}
	else if (strequal(self->classname, "weapon_grenadelauncher"))
	{
		if (leave && (other->items & IT_GRENADE_LAUNCHER) )
			return;
		hadammo = other->ammo_rockets;					 
		neww = IT_GRENADE_LAUNCHER;
		other->ammo_rockets = other->ammo_rockets + 5;
	}
	else if (strequal(self->classname, "weapon_lightning"))
	{
		if (leave && (other->items & IT_LIGHTNING))
			return;
		hadammo = other->ammo_rockets;					 
		neww = IT_LIGHTNING;
		other->ammo_cells = other->ammo_cells + 15;
	}
	else
		objerror ("weapon_touch: unknown classname");

/*	sprint (other, PRINT_LOW, "You got the ");
	sprint (other, PRINT_LOW, self->netname);
	sprint (other, PRINT_LOW, "\n");*/
// weapon touch sound
	G_Sound (other, CHAN_ITEM, "weapons/pkup.wav");
//	stuffcmd (other, "bf\n");

	bound_other_ammo (other);

// change to the weapon
	old = other->items;
	other->items = other->items | neww;
	
	if ( WeaponCode(neww) <= w_switch )
	{
		if (self->flags & FL_INWATER)
		{
			if (neww != IT_LIGHTNING)
			{
				Deathmatch_Weapon (other, old, neww);
			}
		}
		else
		{				 
			Deathmatch_Weapon (other, old, neww);
		}
	}

	W_SetCurrentAmmo (other);

	if (leave)
		return;

	if (g_deathmatch->value!=3 || g_deathmatch->value !=5)
	{
	// remove it in single player, or setup for respawning in deathmatch
		self->modelindex = 0;
		self->solid = SOLID_NOT;
		if (g_deathmatch->value != 2)
			self->nextthink = g_time + 30;
		self->think = RespawnItem;
	}
	G_UseTargets(self, other);		// fire all targets / killtargets
}

/*QUAKED weapon_supershotgun (0 .5 .8) (-16 -16 0) (16 16 32)
*/
void SP_weapon_supershotgun (Entity *ent)
{
	if (g_deathmatch->value >= 4)
	{
		remove (ent);
		return;
	}

	G_PrecacheModel ("progs/g_shot.mdl");
	gi.setmodel (ent, "progs/g_shot.mdl");
	ent->effects = EF_ROTATE;
	ent->weapon = IT_SUPER_SHOTGUN;
//	ent->netname = "Double-barrelled Shotgun";
	ent->touch = Touch_Weapon;
	setsize (ent, -16, -16, 0, 16, 16, 56);
	StartItem (ent);
}

/*QUAKED weapon_nailgun (0 .5 .8) (-16 -16 0) (16 16 32)
*/
void SP_weapon_nailgun (Entity *ent)
{
	if (g_deathmatch->value >= 4)
	{
		remove (ent);
		return;
	}

	G_PrecacheModel ("progs/g_nail.mdl");
	gi.setmodel (ent, "progs/g_nail.mdl");
	ent->effects = EF_ROTATE;
	ent->weapon = IT_NAILGUN;
//	ent->netname = "nailgun";
	ent->touch = Touch_Weapon;
	setsize (ent, -16, -16, 0, 16, 16, 56);
	StartItem (ent);
}

/*QUAKED weapon_supernailgun (0 .5 .8) (-16 -16 0) (16 16 32)
*/
void SP_weapon_supernailgun (Entity *ent)
{
	if (g_deathmatch->value >= 4)
	{
		remove (ent);
		return;
	}

	G_PrecacheModel ("progs/g_nail2.mdl");
	gi.setmodel (ent, "progs/g_nail2.mdl");
	ent->effects = EF_ROTATE;
	ent->weapon = IT_SUPER_NAILGUN;
//	ent->netname = "Super Nailgun";
	ent->touch = Touch_Weapon;
	setsize (ent, -16, -16, 0, 16, 16, 56);
	StartItem (ent);
}

/*QUAKED weapon_grenadelauncher (0 .5 .8) (-16 -16 0) (16 16 32)
*/
void SP_weapon_grenadelauncher (Entity *ent)
{
	if (g_deathmatch->value >= 4)
	{
		remove (ent);
		return;
	}

	G_PrecacheModel ("progs/g_rock.mdl");
	gi.setmodel (ent, "progs/g_rock.mdl");
	ent->effects = EF_ROTATE;
	ent->weapon = 3;
//	ent->netname = "Grenade Launcher";
	ent->touch = Touch_Weapon;
	setsize (ent, -16, -16, 0, 16, 16, 56);
	StartItem (ent);
}

/*QUAKED weapon_rocketlauncher (0 .5 .8) (-16 -16 0) (16 16 32)
*/
void SP_weapon_rocketlauncher (Entity *ent)
{
	if (g_deathmatch->value >= 4)
	{
		remove (ent);
		return;
	}

	G_PrecacheModel ("progs/g_rock2.mdl");
	gi.setmodel (ent, "progs/g_rock2.mdl");
	ent->effects = EF_ROTATE;
	ent->weapon = 3;
//	ent->netname = "Rocket Launcher";
	ent->touch = Touch_Weapon;
	setsize (ent, -16, -16, 0, 16, 16, 56);
	StartItem (ent);
}


/*QUAKED weapon_lightning (0 .5 .8) (-16 -16 0) (16 16 32)
*/
void SP_weapon_lightning (Entity *ent)
{
	if (g_deathmatch->value >= 4)
	{
		remove (ent);
		return;
	}

	G_PrecacheModel ("progs/g_light.mdl");
	gi.setmodel (ent, "progs/g_light.mdl");
	ent->effects = EF_ROTATE;
	ent->weapon = 3;
//	ent->netname = "Thunderbolt";
	ent->touch = Touch_Weapon;
	setsize (ent, -16, -16, 0, 16, 16, 56);
	StartItem (ent);
}


/*
===============================================================================

AMMO

===============================================================================
*/

void Touch_Ammo (Entity *self, Entity *other)
{
	if (!other->player || other->health <= 0)
		return;

//	int best = W_BestWeapon(other);


// shotgun
	if (self->weapon == 1)
	{
		if (other->ammo_shells >= 100)
			return;
		other->ammo_shells += self->aflag;
	}

// spikes
	if (self->weapon == 2)
	{
		if (other->ammo_nails >= 200)
			return;
		other->ammo_nails += self->aflag;
	}

//      rockets
	if (self->weapon == 3)
	{
		if (other->ammo_rockets >= 100)
			return;
		other->ammo_rockets += self->aflag;
	}

//      cells
	if (self->weapon == 4)
	{
		if (other->ammo_cells >= 100)
			return;
		other->ammo_cells += self->aflag;
	}

	bound_other_ammo (other);
	
//	sprint (other, PRINT_LOW, "You got the ");
//	sprint (other, PRINT_LOW, self->netname);
//	sprint (other, PRINT_LOW, "\n");
// ammo touch sound
	G_Sound (other, CHAN_ITEM, "weapons/lock4.wav");
//	stuffcmd (other, "bf\n");

// change to a better weapon if appropriate

/*	if ( other->weapon == best )
	{
		other->weapon = W_BestWeapon (other);
		W_SetCurrentAmmo (other);
	}*/

// if changed current ammo, update it
	W_SetCurrentAmmo (other);

// remove it in single player, or setup for respawning in deathmatch
	self->model = "";
	self->solid = SOLID_NOT;
	if (g_deathmatch->value != 2)
		self->nextthink = g_time + 30;

	if (g_deathmatch->value == 3 || g_deathmatch->value == 5)
		self->nextthink = g_time + 15;

	self->think = RespawnItem;

	G_UseTargets (self, other);		// fire all targets / killtargets
}

#define ITEM_BIG	1

/*QUAKED item_shells (0 .5 .8) (0 0 0) (32 32 32) big
*/
void SP_item_shells (Entity *ent)
{
	if (g_deathmatch->value == 4)
	{
		remove (ent);
		return;
	}

	ent->touch = Touch_Ammo;

	if (ent->spawnflags & ITEM_BIG)
	{
		G_PrecacheModel ("maps/b_shell1.bsp");
		gi.setmodel (ent, "maps/b_shell1.bsp");
		ent->aflag = 40;
	}
	else
	{
		G_PrecacheModel ("maps/b_shell0.bsp");
		gi.setmodel (ent, "maps/b_shell0.bsp");
		ent->aflag = 20;
	}
	ent->weapon = 1;
//	ent->netname = "shells";
	setsize (ent, 0, 0, 0, 32, 32, 56);
	StartItem (ent);
}

/*QUAKED item_spikes (0 .5 .8) (0 0 0) (32 32 32) big
*/
void SP_item_spikes (Entity *ent)
{
	if (g_deathmatch->value == 4)
	{
		remove (ent);
		return;
	}

	ent->touch = Touch_Ammo;

	if (ent->spawnflags & ITEM_BIG)
	{
		G_PrecacheModel ("maps/b_nail1.bsp");
		gi.setmodel (ent, "maps/b_nail1.bsp");
		ent->aflag = 50;
	}
	else
	{
		G_PrecacheModel ("maps/b_nail0.bsp");
		gi.setmodel (ent, "maps/b_nail0.bsp");
		ent->aflag = 25;
	}
	ent->weapon = 2;
//	ent->netname = "nails";
	setsize (ent, 0, 0, 0, 32, 32, 56);
	StartItem (ent);

}

/*QUAKED item_rockets (0 .5 .8) (0 0 0) (32 32 32) big
*/
void SP_item_rockets (Entity *ent)
{
	if (g_deathmatch->value == 4)
	{
		remove (ent);
		return;
	}

	ent->touch = Touch_Ammo;

	if (ent->spawnflags & ITEM_BIG)
	{
		G_PrecacheModel ("maps/b_rock1.bsp");
		gi.setmodel (ent, "maps/b_rock1.bsp");
		ent->aflag = 10;
	}
	else
	{
		G_PrecacheModel ("maps/b_rock0.bsp");
		gi.setmodel (ent, "maps/b_rock0.bsp");
		ent->aflag = 5;
	}
	ent->weapon = 3;
//	ent->netname = "rockets";
	setsize (ent, 0, 0, 0, 32, 32, 56);
	StartItem (ent);

}


/*QUAKED item_cells (0 .5 .8) (0 0 0) (32 32 32) big
*/
void SP_item_cells (Entity *ent)
{
	if (g_deathmatch->value == 4)
	{
		remove (ent);
		return;
	}

	ent->touch = Touch_Ammo;

	if (ent->spawnflags & ITEM_BIG)
	{
		G_PrecacheModel ("maps/b_batt1.bsp");
		gi.setmodel (ent, "maps/b_batt1.bsp");
		ent->aflag = 12;
	}
	else
	{
		G_PrecacheModel ("maps/b_batt0.bsp");
		gi.setmodel (ent, "maps/b_batt0.bsp");
		ent->aflag = 6;
	}
	ent->weapon = 4;
//	ent->netname = "cells";
	setsize (ent, 0, 0, 0, 32, 32, 56);
	StartItem (ent);

}


/*QUAKED item_weapon (0 .5 .8) (0 0 0) (32 32 32) shotgun rocket spikes big
DO NOT USE THIS!!!! IT WILL BE REMOVED!
*/
#define WEAPON_SHOTGUN	1
#define WEAPON_ROCKET	2
#define WEAPON_SPIKES	4
#define WEAPON_BIG		8
void SP_item_weapon (Entity *ent)
{
	if (g_deathmatch->value == 4)
	{
		remove (ent);
		return;
	}

	ent->touch = Touch_Ammo;

	if (ent->spawnflags & WEAPON_SHOTGUN)
	{
		if (ent->spawnflags & WEAPON_BIG)
		{
			G_PrecacheModel ("maps/b_shell1.bsp");
			gi.setmodel (ent, "maps/b_shell1.bsp");
			ent->aflag = 40;
		}
		else
		{
			G_PrecacheModel ("maps/b_shell0.bsp");
			gi.setmodel (ent, "maps/b_shell0.bsp");
			ent->aflag = 20;
		}
		ent->weapon = 1;
//		ent->netname = "shells";
	}

	if (ent->spawnflags & WEAPON_SPIKES)
	{
		if (ent->spawnflags & WEAPON_BIG)
		{
			G_PrecacheModel ("maps/b_nail1.bsp");
			gi.setmodel (ent, "maps/b_nail1.bsp");
			ent->aflag = 40;
		}
		else
		{
			G_PrecacheModel ("maps/b_nail0.bsp");
			gi.setmodel (ent, "maps/b_nail0.bsp");
			ent->aflag = 20;
		}
		ent->weapon = 2;
//		ent->netname = "nails";
	}

	if (ent->spawnflags & WEAPON_ROCKET)
	{
		if (ent->spawnflags & WEAPON_BIG)
		{
			G_PrecacheModel ("maps/b_rock1.bsp");
			gi.setmodel (ent, "maps/b_rock1.bsp");
			ent->aflag = 10;
		}
		else
		{
			G_PrecacheModel ("maps/b_rock0.bsp");
			gi.setmodel (ent, "maps/b_rock0.bsp");
			ent->aflag = 5;
		}
		ent->weapon = 3;
//		ent->netname = "rockets";
	}
	
	setsize (ent, 0, 0, 0, 32, 32, 56);
	StartItem (ent);
}


/*
===============================================================================

KEYS

===============================================================================
*/

void Touch_Key (Entity *self, Entity *other)
{
	if (!other->player || other->health <= 0)
		return;
	if (other->items & self->items)
		return;

//	sprint (other, PRINT_LOW, "You got the ");
//	sprint (other, PRINT_LOW, self->netname);
//	sprint (other,PRINT_LOW, "\n");

	G_Sound (other, CHAN_ITEM, self->noise);
//	stuffcmd (other, "bf\n");
	other->items = other->items | self->items;

	self->solid = SOLID_NOT;
	self->model = "";

	G_UseTargets (self, other);		// fire all targets / killtargets
}


void key_setsounds (Entity *self)
{
	if (world->worldtype == 0)
	{
		G_PrecacheSound ("misc/medkey.wav");
		self->noise = "misc/medkey.wav";
	}
	if (world->worldtype == 1)
	{
		G_PrecacheSound ("misc/runekey.wav");
		self->noise = "misc/runekey.wav";
	}
	if (world->worldtype == 2)
	{
		G_PrecacheSound ("misc/basekey.wav");
		self->noise = "misc/basekey.wav";
	}
}

/*QUAKED item_key1 (0 .5 .8) (-16 -16 -24) (16 16 32)
SILVER key
In order for keys to work
you MUST set your maps
world->type to one of the
following:
0: medieval
1: metal
2: base
*/
void SP_item_key1 (Entity *self)
{
	if (world->worldtype == 0)
	{
		G_PrecacheModel ("progs/w_s_key.mdl");
		gi.setmodel (self, "progs/w_s_key.mdl");
//		self->netname = "silver key";
	}
	else if (world->worldtype == 1)
	{
		G_PrecacheModel ("progs/m_s_key.mdl");
		gi.setmodel (self, "progs/m_s_key.mdl");
//		self->netname = "silver runekey";
	}
	else if (world->worldtype == 2)
	{
		G_PrecacheModel ("progs/b_s_key.mdl");
		gi.setmodel (self, "progs/b_s_key.mdl");
//		self->netname = "silver keycard";
	}
	key_setsounds (self);
	self->effects = EF_ROTATE;
	self->touch = Touch_Key;
	self->items = IT_KEY1;
	setsize (self, -16, -16, -24, 16, 16, 32);
	StartItem (self);
}

/*QUAKED item_key2 (0 .5 .8) (-16 -16 -24) (16 16 32)
GOLD key
In order for keys to work
you MUST set your maps
world->type to one of the
following:
0: medieval
1: metal
2: base
*/
void SP_item_key2 (Entity *self)
{
	if (world->worldtype == 0)
	{
		G_PrecacheModel ("progs/w_g_key.mdl");
		gi.setmodel (self, "progs/w_g_key.mdl");
//		self->netname = "gold key";
	}
	if (world->worldtype == 1)
	{
		G_PrecacheModel ("progs/m_g_key.mdl");
		gi.setmodel (self, "progs/m_g_key.mdl");
//		self->netname = "gold runekey";
	}
	if (world->worldtype == 2)
	{
		G_PrecacheModel ("progs/b_g_key.mdl");
		gi.setmodel (self, "progs/b_g_key.mdl");
//		self->netname = "gold keycard";
	}

	key_setsounds (self);
	self->effects = EF_ROTATE;
	self->touch = Touch_Key;
	self->items = IT_KEY2;
	setsize (self, -16, -16, -24, 16, 16, 32);
	StartItem (self);
}



/*
===============================================================================

END OF LEVEL RUNES

===============================================================================
*/

void Touch_Sigil (Entity *self, Entity *other)
{
	if (!other->player || other->health <= 0)
		return;

	G_Centerprint (other, "You got the rune!");

	G_Sound (other, CHAN_ITEM, self->noise);
//	stuffcmd (other, "bf\n");
	self->solid = SOLID_NOT;
	self->model = "";
//	serverflags = serverflags | (self->spawnflags & 15);
	self->classname = "";            // so rune doors won't find it
	
	G_UseTargets (self, other);		// fire all targets / killtargets
}


/*QUAKED item_sigil (0 .5 .8) (-16 -16 -24) (16 16 32) E1 E2 E3 E4
End of level sigil, pick up to end episode and return to jrstart.
*/
void SP_item_sigil (Entity *self)
{
	if (!self->spawnflags)
		objerror ("no spawnflags");

	G_PrecacheSound ("misc/runekey.wav");
	self->noise = "misc/runekey.wav";

	if (self->spawnflags & 1)
	{
		G_PrecacheModel ("progs/end1.mdl");
		gi.setmodel (self, "progs/end1.mdl");
	}
	if (self->spawnflags & 2)
	{
		G_PrecacheModel ("progs/end2.mdl");
		gi.setmodel (self, "progs/end2.mdl");
	}
	if (self->spawnflags & 4)
	{
		G_PrecacheModel ("progs/end3.mdl");
		gi.setmodel (self, "progs/end3.mdl");
	}
	if (self->spawnflags & 8)
	{
		G_PrecacheModel ("progs/end4.mdl");
		gi.setmodel (self, "progs/end4.mdl");
	}
	
	self->effects = EF_ROTATE;
	self->touch = Touch_Sigil;
	setsize (self, -16, -16, -24, 16, 16, 32);
	StartItem (self);
}

/*
===============================================================================

POWERUPS

===============================================================================
*/

void Touch_Powerup (Entity *self, Entity *other)
{
	if (!other->player || other->health <= 0)
		return;

//	sprint (other, PRINT_LOW, "You got the ");
//	sprint (other,PRINT_LOW,  self->netname);
//	sprint (other,PRINT_LOW, "\n");

	if (strequal(self->classname, "item_artifact_invulnerability") ||
		strequal(self->classname, "item_artifact_invisibility"))
		self->nextthink = g_time + 60*5;
	else
		self->nextthink = g_time + 60;
	
	self->think = RespawnItem;

	G_Sound (other, CHAN_VOICE, self->noise);
//	stuffcmd (other, "bf\n");
	self->solid = SOLID_NOT;
	other->items = other->items | self->items;
	self->modelindex = 0;

// do the apropriate action
#if 0	// TODO
	if (self->classname == "item_artifact_envirosuit")
	{
		other->rad_time = 1;
		other->radsuit_finished = time + 30;
	}
	
	if (self->classname == "item_artifact_invulnerability")
	{
		other->invincible_time = 1;
		other->invincible_finished = time + 30;
	}
	
	if (self->classname == "item_artifact_invisibility")
	{
		other->invisible_time = 1;
		other->invisible_finished = time + 30;
	}

	if (self->classname == "item_artifact_super_damage")
	{
		if (deathmatch == 4)
		{
			other->armortype = 0;
			other->armorvalue = 0 * 0.01;
			other->ammo_cells = 0;
		}
		other->super_time = 1;
		other->super_damage_finished = time + 30;
	}       
#endif

	G_UseTargets (self, other);		// fire all targets / killtargets
}



/*QUAKED item_artifact_invulnerability (0 .5 .8) (-16 -16 -24) (16 16 32)
Player is invulnerable for 30 seconds
*/
void SP_item_artifact_invulnerability (Entity *self)
{
	self->touch = Touch_Powerup;

	G_PrecacheModel ("progs/invulner.mdl");
	G_PrecacheSound ("items/protect.wav");
	G_PrecacheSound ("items/protect2.wav");
	G_PrecacheSound ("items/protect3.wav");
	self->noise = "items/protect.wav";
	gi.setmodel (self, "progs/invulner.mdl");
//	self->netname = "Pentagram of Protection";
	self->effects = EF_ROTATE|EF_RED;
	self->items = IT_INVULNERABILITY;
	setsize (self, -16, -16, -24, 16, 16, 32);
	StartItem (self);
}

/*QUAKED item_artifact_envirosuit (0 .5 .8) (-16 -16 -24) (16 16 32)
Player takes no damage from water or slime for 30 seconds
*/
void SP_item_artifact_envirosuit (Entity *self)
{
	self->touch = Touch_Powerup;

	G_PrecacheModel ("progs/suit.mdl");
	G_PrecacheSound ("items/suit.wav");
	G_PrecacheSound ("items/suit2.wav");
	self->noise = "items/suit.wav";
	gi.setmodel (self, "progs/suit.mdl");
	self->effects = EF_ROTATE;
//	self->netname = "Biosuit";
	self->items = IT_SUIT;
	setsize (self, -16, -16, -24, 16, 16, 32);
	StartItem (self);
}


/*QUAKED item_artifact_invisibility (0 .5 .8) (-16 -16 -24) (16 16 32)
Player is invisible for 30 seconds
*/
void SP_item_artifact_invisibility (Entity *self)
{
	self->touch = Touch_Powerup;

	G_PrecacheModel ("progs/invisibl.mdl");
	G_PrecacheSound ("items/inv1.wav");
	G_PrecacheSound ("items/inv2.wav");
	G_PrecacheSound ("items/inv3.wav");
	self->noise = "items/inv1.wav";
	gi.setmodel (self, "progs/invisibl.mdl");
	self->effects = EF_ROTATE;
//	self->netname = "Ring of Shadows";
	self->items = IT_INVISIBILITY;
	setsize (self, -16, -16, -24, 16, 16, 32);
	StartItem (self);
}


/*QUAKED item_artifact_super_damage (0 .5 .8) (-16 -16 -24) (16 16 32)
The next attack from the player will do 4x damage
*/
void SP_item_artifact_super_damage (Entity *self)
{
	self->touch = Touch_Powerup;

	G_PrecacheModel ("progs/quaddama.mdl");
	G_PrecacheSound ("items/damage.wav");
	G_PrecacheSound ("items/damage2.wav");
	G_PrecacheSound ("items/damage3.wav");
	self->noise = "items/damage.wav";
	gi.setmodel (self, "progs/quaddama.mdl");
//	if (deathmatch == 4)
//		self->netname = "OctaPower";	
//	else
//		self->netname = "Quad Damage";
	self->items = IT_QUAD;
	self->effects = EF_ROTATE|EF_BLUE;
	setsize (self, -16, -16, -24, 16, 16, 32);
	StartItem (self);
}



/*
===============================================================================

PLAYER BACKPACKS

===============================================================================
*/

void BackpackTouch (Entity *self, Entity *other)
{
#if 0		// TODO
	char	*s;
	int		best, old, new;
	int		acount;
	int		b_switch;

	if (!other->player || other->health <= 0)
		return;
		
	if (deathmatch == 4)
		if (other->invincible_time > 0)
			return;

	if ((stof(infokey(other,"b_switch"))) == 0)
		b_switch = 8;
	else
		b_switch = stof(infokey(other,"b_switch"));
	

	acount = 0;
	sprint (other, PRINT_LOW, "You get ");
 
	if (deathmatch == 4)
	{       
		other->health = other->health + 10;
		sprint (other, PRINT_LOW, "10 additional health\n");
		if ((other->health > 250) && (other->health < 300))
			G_Sound (other, CHAN_ITEM, "items/protect3.wav");
		else
			G_Sound (other, CHAN_ITEM, "weapons/lock4.wav");
		stuffcmd (other, "bf\n");
		remove(self);

		if (other->health >299)
		{               
			if (other->invincible_time != 1)
			{                       
				other->invincible_time = 1;
				other->invincible_finished = time + 30;
				other->items = other->items | IT_INVULNERABILITY;
				
				other->super_time = 1;
				other->super_damage_finished = time + 30;
				other->items = other->items | IT_QUAD;

				other->ammo_cells = 0;		
		
	
				G_Sound (other, CHAN_VOICE, "boss1/sight1.wav");
				stuffcmd (other, "bf\n");               
				bprint (other->netname);
				bprint (" attains bonus powers!!!\n");
			}
		}
		self = other;
		return;
	}
	if (self->items)
		if ((other->items & self->items) == 0)
		{
			acount = 1;
			sprint (other, PRINT_LOW, "the ");
			sprint (other, PRINT_LOW, self->netname);
		}
 
// if the player was using his best weapon, change up to the new one if better          
	stemp = self;
	self = other;
	best = W_BestWeapon();
	self = stemp;

// change weapons
	other->ammo_shells = other->ammo_shells + self->ammo_shells;
	other->ammo_nails = other->ammo_nails + self->ammo_nails;
	other->ammo_rockets = other->ammo_rockets + self->ammo_rockets;
	other->ammo_cells = other->ammo_cells + self->ammo_cells;

	new = self->items;
	if (!new)
		new = other->weapon;
	old = other->items;
	other->items = other->items | self->items;
	
	bound_other_ammo (other);

	if (self->ammo_shells)
	{
		if (acount)
			sprint(other, PRINT_LOW, ", ");
		acount = 1;
		s = ftos(self->ammo_shells);
		sprint (other, PRINT_LOW, s);
		sprint (other, PRINT_LOW, " shells");
	}
	if (self->ammo_nails)
	{
		if (acount)
			sprint(other, PRINT_LOW, ", ");
		acount = 1;
		s = ftos(self->ammo_nails);
		sprint (other, PRINT_LOW, s);
		sprint (other, PRINT_LOW, " nails");
	}
	if (self->ammo_rockets)
	{
		if (acount)
			sprint(other, PRINT_LOW, ", ");
		acount = 1;
		s = ftos(self->ammo_rockets);
		sprint (other, PRINT_LOW, s);
		sprint (other, PRINT_LOW, " rockets");
	}
	if (self->ammo_cells)
	{
		if (acount)
			sprint(other, PRINT_LOW, ", ");
		acount = 1;
		s = ftos(self->ammo_cells);
		sprint (other, PRINT_LOW, s);
		sprint (other,PRINT_LOW, " cells");
	}
	
	if ( (deathmatch==3 || deathmatch == 5) & ( (WeaponCode(new)==6) || (WeaponCode(new)==7) ) & (other->ammo_rockets < 5) )
		other->ammo_rockets = 5;

	sprint (other, PRINT_LOW, "\n");
// backpack touch sound
	G_Sound (other, CHAN_ITEM, "weapons/lock4.wav");
	stuffcmd (other, "bf\n");

	remove(self);
	self = other;
	
// change to the weapon
	
	
	if ( WeaponCode(new) <= b_switch )
	{
		if (self->flags & FL_INWATER)
		{
			if (new != IT_LIGHTNING)
			{
				Deathmatch_Weapon (old, new);
			}
		}
		else
		{                
			Deathmatch_Weapon (old, new);
		}
	}

	W_SetCurrentAmmo ();
#endif
}

/*
===============
DropBackpack
===============
*/
void DropBackpack (Entity *self)
{
	Entity *item;

	if (!(self->ammo_shells + self->ammo_nails + self->ammo_rockets + self->ammo_cells))
		return; // nothing in it

	item = G_Spawn();
	item->origin = self->origin;
	item->origin[2] -= 24;
	
	item->items = self->weapon;
#if 0 // TODO
	if (item->items == IT_AXE)
		item->netname = "Axe";
	else if (item->items == IT_SHOTGUN)
		item->netname = "Shotgun";
	else if (item->items == IT_SUPER_SHOTGUN)
		item->netname = "Double-barrelled Shotgun";
	else if (item->items == IT_NAILGUN)
		item->netname = "Nailgun";
	else if (item->items == IT_SUPER_NAILGUN)
		item->netname = "Super Nailgun";
	else if (item->items == IT_GRENADE_LAUNCHER)
		item->netname = "Grenade Launcher";
	else if (item->items == IT_ROCKET_LAUNCHER)
		item->netname = "Rocket Launcher";
	else if (item->items == IT_LIGHTNING)
		item->netname = "Thunderbolt";
	else
		item->netname = "";
#endif

	item->ammo_shells = self->ammo_shells;
	item->ammo_nails = self->ammo_nails;
	item->ammo_rockets = self->ammo_rockets;
	item->ammo_cells = self->ammo_cells;

	item->velocity[0] = 300;
	item->velocity[1] = (rand() % 200) - 100;
	item->velocity[2] = (rand() % 200) - 100;
	
	item->flags = FL_ITEM;
	item->solid = SOLID_TRIGGER;
	item->movetype = MOVETYPE_TOSS;
	item->touch = BackpackTouch;
	gi.setmodel (item, "progs/backpack.mdl");
	setsize (item, -16, -16, 0, 16, 16, 56);
	G_LinkEntity (item);
	
	item->nextthink = g_time + 120;    // remove after 2 minutes
	item->think = G_FreeEntity;
}
