// g_spawn.c

#include "g_local.h"


#define	SPAWNFLAG_NOT_EASY			256
#define	SPAWNFLAG_NOT_MEDIUM		512
#define	SPAWNFLAG_NOT_HARD			1024
#define	SPAWNFLAG_NOT_DEATHMATCH	2048


field_t fields[] = {
	{"classname", FOFS(classname), F_STRING},
	{"spawnflags", FOFS(spawnflags), F_INT},
	{"model", FOFS(model), F_STRING},
	{"origin", FOFS(origin), F_VECTOR},
	{"angles", FOFS(angles), F_ANGLES},
	{"angle", FOFS(angles), F_ANGLEHACK},
	{"mangle", FOFS(mangle), F_ANGLES},
	{"targetname", FOFS(targetname), F_STRING},
	{"target", FOFS(target), F_STRING},
	{"killtarget", FOFS(killtarget), F_STRING},
	{"delay", FOFS(delay), F_FLOAT},
	{"height", FOFS(height), F_FLOAT},
	{"speed", FOFS(speed), F_FLOAT},
	{"wait", FOFS(wait), F_FLOAT},
	{"lip", FOFS(lip), F_FLOAT},
	{"dmg", FOFS(dmg), F_FLOAT},
	{"items", FOFS(items), F_INT},
	{"message", FOFS(message), F_STRING},
	{"health", FOFS(health), F_INT},
	{"sounds", FOFS(sounds), F_INT},
	{"noise", FOFS(noise), F_STRING},
	{"map", FOFS(map), F_STRING},
	{"worldtype", FOFS(worldtype), F_INT},
	{"style", FOFS(style), F_INT},
	{"light", 0, F_IGNORE},
	{"wad", 0, F_IGNORE},

	{NULL}
};


struct spawn_t {
	char	*name;
	void	(*spawn)(Entity *ent);
};

//-------------------------------------------------------
void SP_info_player_start (Entity *ent);
void SP_info_player_start2 (Entity *ent);
void SP_info_player_deathmatch (Entity *ent);
void SP_info_player_coop (Entity *ent);
void SP_info_intermission (Entity *ent);
void SP_trigger_changelevel (Entity *ent);
//-------------------------------------------------------
void SP_trigger_multiple (Entity *ent);
void SP_trigger_once (Entity *ent);
void SP_trigger_relay (Entity *ent);
void SP_trigger_secret (Entity *ent);
void SP_trigger_teleport (Entity *ent);
void SP_trigger_push (Entity *ent);
void SP_info_teleport_destination (Entity *ent);
//-------------------------------------------------------
void SP_func_door (Entity *ent);
void SP_func_door_secret (Entity *ent);
void SP_func_plat (Entity *ent);
void SP_func_train (Entity *ent);
void SP_func_button (Entity *ent);
void SP_misc_teleporttrain (Entity *ent);
void SP_path_corner (Entity *ent);
//-------------------------------------------------------
void SP_item_health (Entity *ent);
void SP_item_armor1 (Entity *ent);
void SP_item_armor2 (Entity *ent);
void SP_item_armorInv (Entity *ent);
void SP_weapon_supershotgun (Entity *ent);
void SP_weapon_nailgun (Entity *ent);
void SP_weapon_supernailgun (Entity *ent);
void SP_weapon_grenadelauncher (Entity *ent);
void SP_weapon_rocketlauncher (Entity *ent);
void SP_weapon_lightning (Entity *ent);
void SP_item_shells (Entity *ent);
void SP_item_spikes (Entity *ent);
void SP_item_rockets (Entity *ent);
void SP_item_cells (Entity *ent);
void SP_item_weapon (Entity *ent);
void SP_item_key1 (Entity *ent);
void SP_item_key2 (Entity *ent);
void SP_item_sigil (Entity *ent);
void SP_item_artifact_invulnerability (Entity *ent);
void SP_item_artifact_envirosuit (Entity *ent);
void SP_item_artifact_invisibility (Entity *ent);
void SP_item_artifact_super_damage (Entity *ent);
//-------------------------------------------------------
void SP_light (Entity *ent);
void SP_light_fluoro (Entity *ent);
void SP_light_fluorospark (Entity *ent);
void SP_light_globe (Entity *ent);
void SP_light_torch_small_walltorch (Entity *ent);
void SP_light_flame_large_yellow (Entity *ent);
void SP_light_flame_small_yellow (Entity *ent);
void SP_light_flame_small_white (Entity *ent);
void SP_info_null (Entity *ent);
void SP_info_notnull (Entity *ent);
void SP_func_wall (Entity *ent);
void SP_func_illusionary (Entity *ent);
void SP_func_episodegate (Entity *ent);
void SP_func_bossgate (Entity *ent);
void SP_ambient_suck_wind (Entity *ent);
void SP_ambient_drone (Entity *ent);
void SP_ambient_flouro_buzz (Entity *ent);
void SP_ambient_drip (Entity *ent);
void SP_ambient_comp_hum (Entity *ent);
void SP_ambient_thunder (Entity *ent);
void SP_ambient_light_buzz (Entity *ent);
void SP_ambient_swamp1 (Entity *ent);
void SP_ambient_swamp2 (Entity *ent);
//-------------------------------------------------------
void SP_worldspawn (Entity *ent);
//-------------------------------------------------------

spawn_t spawns[] = {
	{"info_player_start", SP_info_player_start},
	{"info_player_start2", SP_info_player_start2},
	{"info_player_deathmatch", SP_info_player_deathmatch},
	{"info_player_coop", SP_info_player_coop},
	{"info_intermission", SP_info_intermission},
	{"trigger_changelevel", SP_trigger_changelevel},
//-------------------------------------------------------
	{"trigger_multiple", SP_trigger_multiple},
	{"trigger_once", SP_trigger_once},
	{"trigger_relay", SP_trigger_relay},
	{"trigger_secret", SP_trigger_secret},
	{"trigger_teleport", SP_trigger_teleport},
	{"trigger_push", SP_trigger_push},
	{"info_teleport_destination", SP_info_teleport_destination},
//-------------------------------------------------------
	{"func_door", SP_func_door},
	{"func_door_secret", SP_func_door_secret},
	{"func_plat", SP_func_plat},
	{"func_train", SP_func_train},
	{"func_button", SP_func_button},
	{"misc_teleporttrain", SP_misc_teleporttrain},
	{"path_corner", SP_path_corner},
//-------------------------------------------------------
	{"item_health", SP_item_health},
	{"item_armor1", SP_item_armor1},
	{"item_armor2", SP_item_armor2},
	{"item_armorInv", SP_item_armorInv},
	{"weapon_supershotgun", SP_weapon_supershotgun},
	{"weapon_nailgun", SP_weapon_nailgun},
	{"weapon_supernailgun", SP_weapon_supernailgun},
	{"weapon_grenadelauncher", SP_weapon_grenadelauncher},
	{"weapon_rocketlauncher", SP_weapon_rocketlauncher},
	{"weapon_lightning", SP_weapon_lightning},
	{"item_shells", SP_item_shells},
	{"item_spikes", SP_item_spikes},
	{"item_rockets", SP_item_rockets},
	{"item_cells", SP_item_cells},
	{"item_weapon", SP_item_weapon},
	{"item_key1", SP_item_key1},
	{"item_key2", SP_item_key2},
	{"item_sigil", SP_item_sigil},
	{"item_artifact_invulnerability", SP_item_artifact_invulnerability},
	{"item_artifact_envirosuit", SP_item_artifact_envirosuit},
	{"item_artifact_invisibility", SP_item_artifact_invisibility},
	{"item_artifact_super_damage", SP_item_artifact_super_damage},
//-------------------------------------------------------
	{"light", SP_light},
	{"light_fluoro", SP_light_fluoro},
	{"light_fluorospark", SP_light_fluorospark},
	{"light_globe", SP_light_globe},
	{"light_torch_small_walltorch", SP_light_torch_small_walltorch},
	{"light_flame_large_yellow", SP_light_flame_large_yellow},
	{"light_flame_small_yellow", SP_light_flame_small_yellow},
	{"light_flame_small_white", SP_light_flame_small_white},
	{"info_null", SP_info_null},
	{"info_notnull", SP_info_notnull},
	{"func_wall", SP_func_wall},
	{"func_illusionary", SP_func_illusionary},
	{"func_episodegate", SP_func_episodegate},
	{"func_bossgate", SP_func_bossgate},
	{"ambient_suck_wind", SP_ambient_suck_wind},
	{"ambient_drone", SP_ambient_drone},
	{"ambient_fluoro_buzz", SP_ambient_flouro_buzz},
	{"ambient_drip", SP_ambient_drip},
	{"ambient_comp_hum", SP_ambient_comp_hum},
	{"ambient_thunder", SP_ambient_thunder},
	{"ambient_light_buzz", SP_ambient_light_buzz},
	{"ambient_swamp1", SP_ambient_swamp1},
	{"ambient_swamp2", SP_ambient_swamp2},
//-------------------------------------------------------
	{"monster_ogre", G_FreeEntity},
	{"monster_demon1", G_FreeEntity},
	{"monster_shambler", G_FreeEntity},
	{"monster_knight", G_FreeEntity},
	{"monster_army", G_FreeEntity},
	{"monster_wizard", G_FreeEntity},
	{"monster_dog", G_FreeEntity},
	{"monster_zombie", G_FreeEntity},
	{"monster_boss", G_FreeEntity},
	{"monster_tarbaby", G_FreeEntity},
	{"monster_hell_knight", G_FreeEntity},
	{"monster_fish", G_FreeEntity},
	{"monster_shalrath", G_FreeEntity},
	{"monster_enforcer", G_FreeEntity},
	{"monster_oldone", G_FreeEntity},
	{"event_lightning", G_FreeEntity},
	{"trigger_monsterjump", G_FreeEntity},
//-------------------------------------------------------
	{"worldspawn", SP_worldspawn},

	{NULL, NULL}
};

/*
=================
G_Spawn

Allocates a new edict.
Try to avoid reusing an entity that was recently freed, because it
can cause the client to think the entity morphed into something else
instead of being removed and recreated, which can cause interpolated
angles and bad trails.
=================
*/
Entity *G_Spawn (void)
{
	int			i;
	Entity		*e;

	for (i=MAX_CLIENTS+1 ; i<globals.num_edicts ; i++)
	{
		e = &g_edicts[i];
		// the first couple seconds of server time can involve a lot of
		// freeing and allocating, so relax the replacement policy
		if (!e->inuse && ( e->freetime < 2 || g_time - e->freetime > 0.5 ) )
		{
			memset (e, 0, sizeof(*e));
			e->inuse = true;
			return e;
		}
	}
	
	if (i == MAX_EDICTS)
	{
		Com_Printf ("WARNING: G_Spawn: no free edicts\n");
		i--;	// step on whatever is the last edict
		e = &g_edicts[i];;
		G_UnlinkEntity (e);
	}
	else
		globals.num_edicts++;

	e = &g_edicts[i];
	memset (e, 0, sizeof(*e));
	e->inuse = true;

	return e;
}

/*
=================
G_FreeEntity

Marks the edict as free
FIXME: walk all entities and NULL out references to this entity
=================
*/
void G_FreeEntity (Entity *ent)
{
	G_UnlinkEntity (ent);		// unlink from world bsp

	memset (ent, 0, sizeof(*ent));
	ent->freetime = g_time;
}


/*
===============
ED_CallSpawn

Finds the spawn function for the entity and calls it
===============
*/
void ED_CallSpawn (Entity *ent)
{
	spawn_t	*s;

	if (!ent->classname)
	{
		Com_DPrintf ("ED_CallSpawn: NULL classname\n");
		G_FreeEntity (ent);
		return;
	}

	// check spawn functions
	for (s=spawns ; s->name ; s++)
	{
		if (strequal(s->name, ent->classname))
		{	// found it
			s->spawn (ent);
			gi.FlushSignon ();
			return;
		}
	}

	Com_DPrintf ("No spawn function for: %s\n", ent->classname);
	G_FreeEntity (ent);
}


/*
=============
ED_NewString
=============
*/
char *ED_NewString (char *string)
{
	char	*newb, *new_p;
	int		i,l;
	
	l = strlen(string) + 1;

	newb = (char *) gi.Z_TagMalloc (l, TAG_LEVEL);

	new_p = newb;

	for (i=0 ; i< l ; i++)
	{
		if (string[i] == '\\' && i < l-1)
		{
			i++;
			if (string[i] == 'n')
				*new_p++ = '\n';
			else
				*new_p++ = '\\';
		}
		else
			*new_p++ = string[i];
	}
	
	return newb;
}


/*
===============
ED_ParseField

Takes a key/value pair and sets the binary values
in an edict
===============
*/
void ED_ParseField (char *key, char *value, Entity *ent)
{
	field_t	*f;
	byte	*b;
	Vec3	vec;

	for (f=fields ; f->name ; f++)
	{
		if (striequal(f->name, key))
		{	// found it
			b = (byte *)ent;

			switch (f->type)
			{
			case F_STRING:
				*(char **)(b+f->ofs) = ED_NewString (value);
				break;
			case F_VECTOR:
				sscanf (value, "%f %f %f", &vec[0], &vec[1], &vec[2]);
				((float *)(b+f->ofs))[0] = vec[0];
				((float *)(b+f->ofs))[1] = vec[1];
				((float *)(b+f->ofs))[2] = vec[2];
				break;
			case F_ANGLES:
				sscanf (value, "%f %f %f", &vec[0], &vec[1], &vec[2]);
				// translate 'pitch yaw roll' --> 'yaw pitch roll'
				((float *)(b+f->ofs))[YAW] = vec[1];
				((float *)(b+f->ofs))[PITCH] = vec[0];
				((float *)(b+f->ofs))[ROLL] = vec[2];
				break;
			case F_INT:
				*(int *)(b+f->ofs) = atoi(value);
				break;
			case F_FLOAT:
				*(float *)(b+f->ofs) = atof(value);
				break;
			case F_ANGLEHACK:
				((float *)(b+f->ofs))[YAW] = atof(value);
				((float *)(b+f->ofs))[PITCH] = 0;
				((float *)(b+f->ofs))[ROLL] = 0;
				break;
			case F_IGNORE:
				break;
			}
			return;
		}
	}
	Com_DPrintf ("%s is not a field\n", key);
}


/*
====================
ED_ParseEdict

Parses an edict out of the given string, returning the new position
ed should be a properly initialized empty edict.
Used for initial level load and for savegames.
====================
*/
char *ED_ParseEdict (char *data, Entity *ent)
{
	qboolean	init;
	char		keyname[256];

	init = false;

// go through all the dictionary pairs
	while (1)
	{	
	// parse key
		data = COM_Parse (data);
		if (com_token[0] == '}')
			break;
		if (!data)
			gi.error ("ED_ParseEntity: EOF without closing brace");
			
		strcpy (keyname, com_token);
		
	// parse value	
		data = COM_Parse (data);
		if (!data)
			gi.error ("ED_ParseEntity: EOF without closing brace");

		if (com_token[0] == '}')
			gi.error ("ED_ParseEntity: closing brace without data");

		init = true;	

// keynames with a leading underscore are used for utility comments,
// and are immediately discarded by quake
		if (keyname[0] == '_')
			continue;

		ED_ParseField (keyname, com_token, ent);
	}

	if (!init)
		memset (ent, 0, sizeof(*ent));

	return data;
}



/*
================
G_SpawnEntities

Creates a server's entity / program execution context by
parsing textual entity definitions out of an ent file.
================
*/
void G_SpawnEntities (char *mapname, char *entstring)
{	
	char		*data;
	Entity		*ent;
	int			inhibit;

	// Clear game state
	memset (g_edicts, 0, MAX_EDICTS*sizeof(Entity)); // FIXME: don't clear the world?
	globals.num_edicts = MAX_CLIENTS + 1;
	g_time = 0;

	memset (g_stats, 0, sizeof(g_stats));
	for (int i = 0; i < MAX_CLIENTS; i++) {
		g_edicts[i].stats = &g_stats[i];
	}

	current_skill = bound (g_skill->value, 0, 3);
	gi.Cvar_Set (g_skill, va("%d", (int)current_skill));

	gi.Z_FreeTags (TAG_LEVEL);

//==================================================================

	data = entstring;
	ent = NULL;
	inhibit = 0;
	
// parse ents
	while (1)
	{
// parse the opening brace	
		data = COM_Parse (data);
		if (!data)
			break;
		if (com_token[0] != '{')
			gi.error ("ED_LoadFromFile: found %s when expecting {", com_token);

		if (!ent)
			ent = &g_edicts[0];
		else
			ent = G_Spawn();
		data = ED_ParseEdict (data, ent);
		
// remove things from different skill levels or deathmatch
		if (g_deathmatch->value)
		{
			if (((int)ent->spawnflags & SPAWNFLAG_NOT_DEATHMATCH))
			{
				G_FreeEntity (ent);
				inhibit++;
				continue;
			}
		}
		else if ((current_skill == 0 && ((int)ent->spawnflags & SPAWNFLAG_NOT_EASY))
				|| (current_skill == 1 && ((int)ent->spawnflags & SPAWNFLAG_NOT_MEDIUM))
				|| (current_skill >= 2 && ((int)ent->spawnflags & SPAWNFLAG_NOT_HARD)) )
		{
			G_FreeEntity (ent);
			inhibit++;
			continue;
		}

		ED_CallSpawn (ent);
	}	

	Com_DPrintf ("%i entities inhibited\n", inhibit);
}

/*
=================
SP_worldspawn
=================
*/
void SP_worldspawn (Entity *ent)
{
	extern int	force_retouch;

	force_retouch = 2;	// FIXME: get rid of this

//	ent->model = sv.modelname;	// FIXME
	ent->modelindex = 1;
	ent->movetype = MOVETYPE_PUSH;
	ent->solid = SOLID_BSP;

	// setup precaches always needed
	G_PrecacheSound ("items/itembk2.wav");		// item respawn sound
	G_PrecacheSound ("player/plyrjmp8.wav");	// player jump
	G_PrecacheSound ("player/land.wav");		// player landing
	G_PrecacheSound ("player/land2.wav");		// player hurt landing
	G_PrecacheSound ("player/drown1.wav");		// drowning pain
	G_PrecacheSound ("player/drown2.wav");		// drowning pain
	G_PrecacheSound ("player/gasp1.wav");		// gasping for air
	G_PrecacheSound ("player/gasp2.wav");		// taking breath
	G_PrecacheSound ("player/h2odeath.wav");	// drowning death
//	G_PrecacheSound ("demon/dland2.wav");		// monster landing
	G_PrecacheSound ("misc/h2ohit1.wav");		// landing splash

	// player pain sounds
	G_PrecacheSound ("player/pain1.wav");
	G_PrecacheSound ("player/pain2.wav");
	G_PrecacheSound ("player/pain3.wav");
	G_PrecacheSound ("player/pain4.wav");
	G_PrecacheSound ("player/pain5.wav");
	G_PrecacheSound ("player/pain6.wav");

	// player death sounds
	G_PrecacheSound ("player/death1.wav");
	G_PrecacheSound ("player/death2.wav");
	G_PrecacheSound ("player/death3.wav");
	G_PrecacheSound ("player/death4.wav");
	G_PrecacheSound ("player/death5.wav");

	// player gib sounds
	G_PrecacheSound ("player/gib.wav");
	G_PrecacheSound ("player/udeath.wav");
	G_PrecacheSound ("player/tornoff2.wav");

	G_PrecacheSound ("misc/talk.wav");
	G_PrecacheSound ("player/teledth1.wav");	// telefrag
	G_PrecacheSound ("misc/r_tele1.wav");		// teleport sounds
	G_PrecacheSound ("misc/r_tele2.wav");
	G_PrecacheSound ("misc/r_tele3.wav");
	G_PrecacheSound ("misc/r_tele4.wav");
	G_PrecacheSound ("misc/r_tele5.wav");

	G_PrecacheSound ("weapons/lock4.wav");		// ammo pick up
	G_PrecacheSound ("weapons/pkup.wav");		// weapon pick up
//	G_PrecacheSound ("items/armor1.wav");		// armor pick up
	G_PrecacheSound ("items/damage3.wav");

	// weapon sounds
	G_PrecacheSound ("weapons/ax1.wav");		// axe swoosh
	G_PrecacheSound ("weapons/guncock.wav");	// shotgun
	G_PrecacheSound ("weapons/shotgn2.wav");	// super shotgun
	G_PrecacheSound ("weapons/rocket1i.wav");	// nailgun
	G_PrecacheSound ("weapons/spike2.wav"); 	// super nailgun
	G_PrecacheSound ("weapons/sgun1.wav");		// rocket launcher
	G_PrecacheSound ("weapons/grenade.wav");	// grenade launcher
	G_PrecacheSound ("weapons/lhit.wav");		// lightning
	G_PrecacheSound ("weapons/lstart.wav"); 	// lightning start
	G_PrecacheSound ("weapons/bounce.wav"); 	// grenade bounce
	G_PrecacheSound ("weapons/tink1.wav");		// spikes tink (used in c code)
	G_PrecacheSound ("weapons/ric1.wav");		// ricochet (used in c code)
	G_PrecacheSound ("weapons/ric2.wav");		// ricochet (used in c code)
	G_PrecacheSound ("weapons/ric3.wav");		// ricochet (used in c code)
	G_PrecacheSound ("weapons/r_exp3.wav"); 	// new rocket explosion
	G_PrecacheSound ("player/axhit1.wav");		// axe hit meat
	G_PrecacheSound ("player/axhit2.wav");		// axe hit world

	G_PrecacheModel ("progs/player.mdl");
	G_PrecacheModel ("progs/eyes.mdl");

	G_PrecacheModel ("progs/v_axe.mdl");
	G_PrecacheModel ("progs/v_shot.mdl");
	G_PrecacheModel ("progs/v_shot2.mdl");
	G_PrecacheModel ("progs/v_nail.mdl");
	G_PrecacheModel ("progs/v_nail2.mdl");
	G_PrecacheModel ("progs/v_rock.mdl");
	G_PrecacheModel ("progs/v_rock2.mdl");
	G_PrecacheModel ("progs/v_light.mdl");

	G_PrecacheModel ("progs/spike.mdl");
	G_PrecacheModel ("progs/s_spike.mdl");
	G_PrecacheModel ("progs/grenade.mdl");
	G_PrecacheModel ("progs/missile.mdl");
	G_PrecacheModel ("progs/bolt.mdl");              // for lightning gun

	G_PrecacheModel ("progs/backpack.mdl");

	// gibs
	G_PrecacheModel ("progs/h_player.mdl");
	G_PrecacheModel ("progs/gib1.mdl");
	G_PrecacheModel ("progs/gib2.mdl");
	G_PrecacheModel ("progs/gib3.mdl");
//	G_PrecacheModel ("progs/zom_gib.mdl");
}
