// g_trigger.c

#include "g_local.h"
#include "../common/pmove.h"	// for PMF_ONGROUND

int found_secrets = 0;
int total_secrets = 0;

#define SPAWNFLAG_NOMESSAGE		1
#define SPAWNFLAG_NOTOUCH		1


void InitTrigger (Entity *ent)
{
// trigger angles are used for one-way touches.  An angle of 0 is assumed
// to mean no restrictions, so use a yaw of 360 instead.
	if (!ent->angles.IsNull())
		G_SetMovedir (ent->angles, ent->movedir);

	ent->solid = SOLID_TRIGGER;
	ent->movetype = MOVETYPE_NONE;
	gi.setmodel(ent, ent->model);		// set size and link into world

	// FIXME
	ent->modelindex = 0;
	ent->model = "";
}

// the wait time has passed, so set back up for another activation
void multi_wait (Entity *ent)
{
	if (ent->max_health)
	{
		ent->health = ent->max_health;
		ent->takedamage = DAMAGE_YES;
		ent->solid = SOLID_BBOX;
	}
}


// the trigger was just touched/killed/used
// ent->enemy should be set to the activator so it can be held through a delay
// so wait for the delay time before firing
void multi_trigger (Entity *ent)
{
	if (ent->nextthink > g_time)
	{
		return;		// already been triggered
	}

	if (strequal(ent->classname, "trigger_secret"))
	{
		if (!ent->enemy->player)
			return;
		found_secrets = found_secrets + 1;
//		WriteByte (MSG_ALL, SVC_FOUNDSECRET);
	}

	if (ent->noise)
		G_Sound (ent, CHAN_VOICE, ent->noise);

// don't trigger again until reset
	ent->takedamage = DAMAGE_NO;

	G_UseTargets (ent, ent->enemy);

	if (ent->wait > 0)	
	{
		ent->think = multi_wait;
		ent->nextthink = g_time + ent->wait;
	}
	else
	{	// we can't just remove (self) here, because this is a touch function
		// called wheil C code is looping through area links...
		ent->touch = NULL;
		ent->nextthink = g_time + 0.1;
		ent->think = G_FreeEntity;
	}
}

void multi_killed (Entity *self, Entity *inflictor, Entity *attacker)
{
	self->enemy = attacker;
	multi_trigger (self);
}

void multi_use (Entity *self, Entity *other, Entity *activator)
{
	self->enemy = activator;
	multi_trigger (self);
}

void multi_touch (Entity *ent, Entity *other)
{
	Vec3	forward;

	if (!other->player)
		return;
	
// if the trigger has an angles field, check player's facing direction
	if (!ent->movedir.IsNull())
	{
		AngleVectors (other->angles, forward);
		if (DotProduct(forward, ent->movedir) < 0)
			return;		// not facing the right way
	}
	
	ent->enemy = other;
	multi_trigger (ent);
}

/*QUAKED trigger_multiple (.5 .5 .5) ? notouch
Variable sized repeatable trigger.  Must be targeted at one or more entities.  If "health" is set, the trigger must be killed to activate each time.
If "delay" is set, the trigger waits some time after activating before firing.
"wait" : Seconds between triggerings. (.2 default)
If notouch is set, the trigger is only fired by other entities, not by touching.
NOTOUCH has been obsoleted by trigger_relay!
sounds
1)	secret
2)	beep beep
3)	large switch
4)
set "message" to text string
*/
void SP_trigger_multiple (Entity *ent)
{
	if (ent->sounds == 1)
	{
		G_PrecacheSound ("misc/secret.wav");
		ent->noise = "misc/secret.wav";
	}
	else if (ent->sounds == 2)
	{
		G_PrecacheSound ("misc/talk.wav");
		ent->noise = "misc/talk.wav";
	}
	else if (ent->sounds == 3)
	{
		G_PrecacheSound ("misc/trigger1.wav");
		ent->noise = "misc/trigger1.wav";
	}
	
	if (!ent->wait)
		ent->wait = 0.2;
	ent->use = multi_use;

	InitTrigger (ent);

	if (ent->health)
	{
		if (ent->spawnflags & SPAWNFLAG_NOTOUCH)
			objerror ("health and notouch don't make sense\n");
		ent->max_health = ent->health;
		ent->die = multi_killed;
		ent->takedamage = DAMAGE_YES;
		ent->solid = SOLID_BBOX;
		G_LinkEntity (ent);
	}
	else
	{
		if ( !(ent->spawnflags & SPAWNFLAG_NOTOUCH) )
		{
			ent->touch = multi_touch;
		}
	}
}


/*QUAKED trigger_once (.5 .5 .5) ? notouch
Variable sized trigger. Triggers once, then removes itent->  You must set the key "target" to the name of another object in the level that has a matching
"targetname".  If "health" is set, the trigger must be killed to activate.
If notouch is set, the trigger is only fired by other entities, not by touching.
if "killtarget" is set, any objects that have a matching "target" will be removed when the trigger is fired.
if "angle" is set, the trigger will only fire when someone is facing the direction of the angle.  Use "360" for an angle of 0.
sounds
1)	secret
2)	beep beep
3)	large switch
4)
set "message" to text string
*/
void SP_trigger_once (Entity *ent)
{
	ent->wait = -1;
	SP_trigger_multiple (ent);
}

//=============================================================================

/*QUAKED trigger_relay (.5 .5 .5) (-8 -8 -8) (8 8 8)
This fixed size trigger cannot be touched, it can only be fired by other events.  It can contain killtargets, targets, delays, and messages.
*/
void trigger_relay_use (Entity *self, Entity *other, Entity *activator)
{
	G_UseTargets (self, activator);
}

void SP_trigger_relay (Entity *ent)
{
	ent->use = trigger_relay_use;
}


//=============================================================================

/*QUAKED trigger_secret (.5 .5 .5) ?
secret counter trigger
sounds
1)	secret
2)	beep beep
3)
4)
set "message" to text string
*/
void SP_trigger_secret (Entity *ent)
{
	total_secrets = total_secrets + 1;
	ent->wait = -1;
	if (!ent->message)
		ent->message = "You found a secret area!";
	if (!ent->sounds)
		ent->sounds = 1;
	
	if (ent->sounds == 1)
	{
		G_PrecacheSound ("misc/secret.wav");
		ent->noise = "misc/secret.wav";
	}
	else if (ent->sounds == 2)
	{
		G_PrecacheSound ("misc/talk.wav");
		ent->noise = "misc/talk.wav";
	}

	SP_trigger_multiple (ent);
}


/*
==============================================================================

TELEPORT TRIGGERS

==============================================================================
*/

#define PLAYER_ONLY		1
#define SILENT			2

void play_teleport (Entity *self)
{
	int		i;
	char	*str;

	i = rand() % 5;
	switch (i)
	{
		case 0: str = "misc/r_tele1.wav"; break;
		case 1: str = "misc/r_tele2.wav"; break;
		case 2: str = "misc/r_tele3.wav"; break;
		case 3: str = "misc/r_tele4.wav"; break;
		default: str = "misc/r_tele5.wav";
	}

	G_Sound (self, CHAN_VOICE, str);
	remove (self);
}

void spawn_tfog (const Vec3 &org)
{
	Entity	*s;

	s = G_Spawn();
	s->origin = org;
	s->nextthink = g_time + 0.2;
	s->think = play_teleport;

	gi.event_txyz (TE_TELEPORT, org, MULTICAST_PHS);
}


void tdeath_touch (Entity *self, Entity *other)
{
//	Entity *other2;

	if (other == self->owner)
		return;

/*
// frag anyone who teleports in on top of an invincible player
	if (other->player)
	{
		if (other->invincible_finished > g_time &&
			self->owner->invincible_finished > g_time) {
			self->classname = "teledeath3";
			other->invincible_finished = 0;
			self->owner->invincible_finished = 0;
			T_Damage (other, self, self, 50000);
			other2 = self->owner;
			self->owner = other;
			T_Damage (other2, self, self, 50000);
		}
			
		if (other->invincible_finished > g_time)
		{
			T_Damage (self->owner, self, self, 50000, DT_TELEDEATH2);
			return;
		}
		
	}
*/

	if (other->health)
	{
		G_Damage (other, self, self, 50000, DT_TELEDEATH);
	}
}


void spawn_tdeath (const Vec3 &org, Entity *death_owner)
{
	Entity *death;
	extern int force_retouch;

	death = G_Spawn();

	death->classname = "teledeath";
	death->movetype = MOVETYPE_NONE;
	death->solid = SOLID_TRIGGER;

	death->mins[0] = death_owner->mins[0] - 1;
	death->mins[1] = death_owner->mins[1] - 1;
	death->mins[2] = death_owner->mins[2] - 1;
	death->maxs[0] = death_owner->maxs[0] + 1;
	death->maxs[1] = death_owner->maxs[1] + 1;
	death->maxs[2] = death_owner->maxs[2] + 1;

	death->origin = org;
	G_LinkEntity (death);

	death->touch = tdeath_touch;
	death->nextthink = g_time + 0.2;
	death->think = G_FreeEntity;
	death->owner = death_owner;
	
	// FIXME
	force_retouch = 2;		// make sure even still objects get hit
}

void teleport_touch (Entity *self, Entity *other)
{
	Vec3	org, forward;

	if (self->targetname)
	{
		if (self->nextthink < g_time)
		{
			return;		// not fired yet
		}
	}

	if ((self->spawnflags & PLAYER_ONLY) && !other->player)
		return;

// only teleport living creatures
// FIXME: make a better check, now that we don't have SOLID_SLIDEBOX?
	if (other->health <= 0 || other->solid != SOLID_BBOX)
		return;

	Entity *t = G_Find (NULL, FOFS(targetname), self->target);
	if (!t) {
		// objerror ("couldn't find target");
		return;
	}

	G_UseTargets (self, other);

// put a tfog where the player was
	spawn_tfog (other->origin);

// spawn a tfog flash in front of the destination
	AngleVectors (t->mangle, forward);
	org = t->origin + forward * 32;

	spawn_tfog (org);
	spawn_tdeath (t->origin, other);

// move the player
	other->viewangles = t->mangle;
	other->angles = other->viewangles;	// hmm
	other->origin = t->origin;
	G_LinkEntity (other);

	other->groundentity = NULL;

	if (other->player)
	{
		other->fixangle = true;		// turn this way immediately
		other->pmove.steptime = 0;
		other->pmove.landtime = 0;
		other->velocity = forward * 300;
	}

}

/*QUAKED info_teleport_destination (.5 .5 .5) (-8 -8 -8) (8 8 32)
This is the destination marker for a teleporter.  It should have a "targetname" field with the same value as a teleporter's "target" field.
*/
void SP_info_teleport_destination (Entity *self)
{
// this does nothing, just serves as a target spot
	self->mangle = self->angles;
	self->angles.Clear();
	self->model = "";
	self->origin[2] += 27;
	if (!self->targetname)
		objerror ("no targetname");
}

void teleport_use (Entity *self, Entity *other, Entity *activator)
{
	extern int force_retouch;

	force_retouch = 2;		// make sure even still objects get hit
//	self->nextthink = time + 0.2;
//	self->think = SUB_Null;
}

/*QUAKED trigger_teleport (.5 .5 .5) ? PLAYER_ONLY SILENT
Any object touching this will be transported to the corresponding info_teleport_destination entity. You must set the "target" field, and create an object with a "targetname" field that matches.

If the trigger_teleport has a targetname, it will only teleport entities when it has been fired.
*/
void SP_trigger_teleport (Entity *self)
{
	InitTrigger (self);

	self->touch = teleport_touch;
	self->use = teleport_use;

	// find the destination 
	if (!self->target)
		objerror ("no target");

	if (!(self->spawnflags & SILENT))
	{
		G_PrecacheSound ("ambience/hum1.wav");
		Vec3 origin = (self->mins + self->maxs) * 0.5;
		G_StaticSound (origin, "ambience/hum1.wav", 0.5, ATTN_STATIC);
	}
}

//============================================================================

const int PUSH_ONCE = 1;

void trigger_push_touch (Entity *self, Entity *other)
{
	if (strequal(other->classname, "grenade"))
		other->velocity = self->speed * self->movedir * 10;
	else if (other->health > 0)
	{
		other->pmove.pm_flags &= ~PMF_ONGROUND;
		other->groundentity = NULL;
		other->velocity = self->speed * self->movedir * 10;
		if (strequal(other->classname, "player"))
		{
			if (other->fly_sound < g_time)
			{
				other->fly_sound = g_time + 1.5;
				G_Sound (other, CHAN_AUTO, "ambience/windfly.wav");
			}
		}
	}

	if (self->spawnflags & PUSH_ONCE)
		remove(self);
}


/*QUAKED trigger_push (.5 .5 .5) ? PUSH_ONCE
Pushes the player
*/
void SP_trigger_push (Entity *self)
{
	InitTrigger (self);
	G_PrecacheSound ("ambience/windfly.wav");
	self->touch = trigger_push_touch;
	if (!self->speed)
		self->speed = 1000;
}
